package de.uni_heidelberg.ise.studentnavigator;

import android.app.Activity;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import de.uhd.ifi.se.feature.annotations.java.Feature;

@Feature("Routenfindung")
public class ElevatorManager {
    private final ImageButton btn2Dimension;
    private final ImageButton btn3Dimension;
    private final ImageButton btnCompass;
    private final ImageButton btnElevatorDown;
    private final ImageButton btnElevatorUp;
    private final MainActivity context;
    private EditText currentFloor;

    public ElevatorManager(Activity context) {
        this.context = (MainActivity) context;
        btnCompass = (ImageButton) context.findViewById(R.id.buttonCompass);
        btn3Dimension = (ImageButton) context.findViewById(R.id.button3Dimension);
        btn2Dimension = (ImageButton) context.findViewById(R.id.button2Dimension);
        btnElevatorDown = (ImageButton) context.findViewById(R.id.buttonDown_Elevator);
        btnElevatorUp = (ImageButton) context.findViewById(R.id.buttonUp_Elevator);
        currentFloor = (EditText) context.findViewById(R.id.editText_Elevator);

        initialize();
    }

    public EditText getCurrentFloor() {
        return currentFloor;
    }

    private void initBtn2Dimension() {
        btn2Dimension.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClicked2DimensionBtn();
            }
        });
    }

    private void initBtn3Dimension() {
        btn3Dimension.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClicked3DimensionBtn();
            }
        });
    }

    private void initBtnCompass() {
        btnCompass.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClickedCompassBtn();
            }
        });
    }

    private void initBtnElevatorDown() {
        btnElevatorDown.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClickedElevArrow(true);
            }
        });
    }

    private void initBtnElevatorUp() {
        btnElevatorUp.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClickedElevArrow(false);
            }
        });
    }

    private void initCurrentFloor() {
        currentFloor.addTextChangedListener(new TextWatcher() {
            @Override
            public void afterTextChanged(Editable e) {
                context.getDeepMap().setLevel(Integer.parseInt(e.toString().isEmpty() ? "0" : e.toString()));
            }

            @Override
            public void beforeTextChanged(CharSequence s, int i, int i1, int i2) {
            }

            @Override
            public void onTextChanged(CharSequence s, int i, int i1, int i2) {
            }
        });
    }

    private void initialize() {
        initBtnCompass();
        initBtn2Dimension();
        initBtn3Dimension();
        initBtnElevatorDown();
        initBtnElevatorUp();
        initCurrentFloor();
    }

    private void onClicked2DimensionBtn() {
        context.getDeepMap().set3DMode(false, true);
        btn2Dimension.setVisibility(View.GONE);
        btn3Dimension.setVisibility(View.VISIBLE);
    }

    private void onClicked3DimensionBtn() {
        context.getDeepMap().set3DMode(true, true);
        btn3Dimension.setVisibility(View.GONE);
        btn2Dimension.setVisibility(View.VISIBLE);
    }

    private void onClickedCompassBtn() {
        if (context.getInitialPerspective() != null)
            context.getDeepMap().resetNorth(true);
    }

    private void onClickedElevArrow(boolean down) {
        final int currentNumber = Integer.parseInt(currentFloor.getText().toString()),
                level = (currentNumber == context.getDeepMap().getMinimumLevel() && down)
                        || (currentNumber == context.getDeepMap().getMaximumLevel() && !down) ? currentNumber
                                : down ? currentNumber -1 : currentNumber + 1;

        currentFloor.setText(String.valueOf(level));
        context.getDeepMap().setLevel(level);
    }

    public void setCurrentFloor(EditText currentFloor) {
        this.currentFloor = currentFloor;
    }
}
