package de.uni_heidelberg.ise.studentnavigator;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.hdm_i.dm.android.mapsdk.Annotation;
import com.hdm_i.dm.android.mapsdk.Annotation.OnAnnotationClickListener;
import com.hdm_i.dm.android.mapsdk.MapView;
import com.j256.ormlite.support.ConnectionSource;

import android.widget.ImageView;
import android.widget.SearchView;
import android.widget.Toast;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.extern.QueryCallback;
import de.uni_heidelberg.ise.studentnavigator.extern.RnvDeparture;
import de.uni_heidelberg.ise.studentnavigator.extern.RnvJourney;
import de.uni_heidelberg.ise.studentnavigator.model.Building;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Person;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.api.DataManager;
import de.uni_heidelberg.ise.studentnavigator.model.api.IDataManager;

@Feature("filtern")
public class FilterManager {
    private final List<Annotation> annotations;
    private final MainActivity context;
    private final IDataManager dbManager;
    private final List<Long> markedFeatures;

    public FilterManager(ConnectionSource con, MainActivity context) {
        this.context = context;
        this.dbManager = new DataManager(con);
        this.markedFeatures = new ArrayList<Long>();
        this.annotations = new ArrayList<Annotation>();
    }

    private void annotateStation(final Station s) {
        final MainActivity context = this.context;
        this.context.opnvManager.queryJourney(s.getHafasID(), new Date(), new ArrayList<String>(), false,
                new QueryCallback<RnvJourney>() {
                    @Override
                    public void onFailure(Exception x) {
                        context.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Toast.makeText(context, StringConstants.noConnection, Toast.LENGTH_SHORT).show();
                            }
                        });
                    }

                    @Override
                    public void onSuccess(final RnvJourney j) {
                        context.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                final Annotation annotation = new Annotation(s.getLongName(),
                                        context.getDeepMap().getFeatureWithId(Long.parseLong(s.getFeatureId())));
                                annotations.add(annotation);
                                final ArrayList<String> subt = new ArrayList<String>();
                                for (final RnvDeparture r : j.getListOfDepartures())
                                    subt.add("[" + r.getLineLabel() + "] ");
                                annotation.setSubtitle(subt.toString());
                                annotation.setClickListener(new OnAnnotationClickListener() {

                                    @Override
                                    public void onAnnotationClick(Annotation arg0) {
                                        context.runOnUiThread(new Runnable() {
                                            @Override
                                            public void run() {
                                                context.getInfoBoxManager().setInfo(s, j);
                                            }
                                        });
                                    }

                                });
                                context.getDeepMap().addAnnotation(annotation);
                            }
                        });
                    }
                });
    }

    private void changeSearchImage(String type) {
        final SearchView searchView = (SearchView) context.findViewById(R.id.search),
                searchView2 = (SearchView) context.findViewById(R.id.search2);
        final int searchImgId = context.getResources().getIdentifier("android:id/search_mag_icon", null, null);
        final ImageView typeIcon = (ImageView) searchView.findViewById(searchImgId),
                typeIcon2 = (ImageView) searchView2.findViewById(searchImgId);
        final int iconId = getIconId(type);
        typeIcon.setImageResource(iconId);
        typeIcon2.setImageResource(iconId);
    }

    private int getIconId(String type) {
        context.setFilter(type);
        return type == "person" ? R.drawable.ic_menu_persons
                : type == "building" ? R.drawable.ic_menu_building
                        : type == "room" ? R.drawable.ic_menu_freerooms
                                : type == "station" ? R.drawable.ic_train : R.drawable.ic_search_icon;
    }

    public void markAllBuildings() {
        unmarkAll();
        for (final Building building : dbManager.getBuildings())
            markFeature(building.getFeatureId());
        changeSearchImage("building");
    }

    public void markAllPersons() {
        unmarkAll();
        for (final Person person : dbManager.getPersons())
            markFeature(person.getRoom().getFeatureId());
        changeSearchImage("person");
    }

    public void markAllRooms() {
        unmarkAll();
        markAllOffices();
        markAllEventLocations();
        markAllToilets();
        changeSearchImage("room");
    }

    private void markAllEventLocations() {
        for (final EventLocation el : dbManager.getEventLocations())
            markFeature(el.getFeatureId());
    }

    private void markAllOffices() {
        for (final Office office : dbManager.getOffices())
            markFeature(office.getFeatureId());
    }

    private void markAllToilets() {
        for (final Toilet toilet : dbManager.getToilets())
            markFeature(toilet.getFeatureId());
    }

    public void markAllStations() {
        unmarkAll();
        for (final Station station : dbManager.getStations()) {
            markFeature(station.getFeatureId());
            annotateStation(station);
        }
        changeSearchImage("station");
    }

    private void markFeature(String featureId) {
        try {
            final Long parsedFeatureId = Long.valueOf(featureId);
            context.getDeepMap().highlightFeatureWithId(parsedFeatureId);
            markedFeatures.add(parsedFeatureId);
        } catch (final NumberFormatException e) {
            System.err.println(e.getMessage());
        }
    }

    public void removeAnnotations() {
        context.getDeepMap().removeAnnotations(annotations);
    }

    public void unmarkAll() {
        for (final Long feature : this.markedFeatures)
            this.context.getDeepMap().unhighlightFeatureWithId(feature);
        changeSearchImage("none");
        this.markedFeatures.clear();
    }
}
