package de.uni_heidelberg.ise.studentnavigator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.hdm_i.dm.android.mapsdk.MapView;
import com.sothree.slidinguppanel.SlidingUpPanelLayout;
import com.sothree.slidinguppanel.SlidingUpPanelLayout.PanelSlideListener;
import com.sothree.slidinguppanel.SlidingUpPanelLayout.PanelState;

import android.app.Activity;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.ToggleButton;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.extern.AsyncRetrieveTweetsTask;
import de.uni_heidelberg.ise.studentnavigator.extern.IAsyncRetrieveTweetsResponse;
import de.uni_heidelberg.ise.studentnavigator.extern.QueryCallback;
import de.uni_heidelberg.ise.studentnavigator.extern.RnvJourney;
import de.uni_heidelberg.ise.studentnavigator.model.Building;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Person;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.api.ISearchable;
import de.uni_heidelberg.ise.studentnavigator.InfoBoxPersonManager;
import twitter4j.Status;

@Feature("Routenfindung")
public class InfoBoxManager implements IAsyncRetrieveTweetsResponse {
    private enum State {
        INFO, MENU, OPNV
    }
    private static final int NUMBER_OF_TWEETS = 5;
    private LinearLayout contentContainer;
    private final MainActivity context;
    private final FilterManager filterManager;
    private String lastFocussedFeature;
    private final SlidingUpPanelLayout layout;
    private ListView listView;
    private LinearLayout menuContainer;
    private View opnvBoxContainer;
    private State state;

    public InfoBoxManager(Activity context) {
        this.context = (MainActivity) context;
        menuContainer = context.findViewById(R.id.menu_container_ref);
        setFilterButtonListeners(menuContainer);
        contentContainer = (LinearLayout) menuContainer.getParent();
        layout = (SlidingUpPanelLayout) context.findViewById(R.id.sliding_layout);
        filterManager = new FilterManager(this.context.getDataBaseHelper().getConnectionSource(), this.context);
        layout.addPanelSlideListener(new PanelSlideListener() {
            @Override
            public void onPanelSlide(View arg0, float arg1) {
            }

            @Override
            public void onPanelStateChanged(View arg0, PanelState arg1, PanelState arg2) {
                if (PanelState.COLLAPSED.equals(arg2)) {
                    setMenuInBox();
                    removeFocusOnLastFeature();
                    if (state == State.OPNV) {
                        resetMenu();
                    }
                }
            }

        });
        state = State.MENU;
    }

    @Feature("Routenfindung")
    private void addBuildingInfo(Building building) {
        setLayoutForSubContainer(R.layout.layout_info_building);

        final TextView buildingTitle = (TextView) context.findViewById(R.id.building_title_label);

        final TextView buildingTitleText = (TextView) context.findViewById(R.id.building_title);

        if (building.getName() != null) {
            buildingTitleText.setText(building.getName());
        } else {
            removeView(buildingTitleText, buildingTitle);
        }

        if (building.getFeatureId() != null) {
            focusOnFeature(building.getFeatureId());
        }
    }

    @Feature("Routenfindung")
    private void addEventLocationInfo(EventLocation eventLocation) {
        setLayoutForSubContainer(R.layout.layout_info_eventlocation);

        final TextView elocationRoom = (TextView) context.findViewById(R.id.elocation_room_label);
        final TextView elocationBuilding = (TextView) context.findViewById(R.id.elocation_building_label);
        final TextView elocationOrganization = (TextView) context.findViewById(R.id.elocation_organization_label);

        final TextView elocationRoomText = (TextView) context.findViewById(R.id.elocation_room);
        final TextView elocationBuildingText = (TextView) context.findViewById(R.id.elocation_building);
        final TextView elocationOrganizationText = (TextView) context.findViewById(R.id.elocation_organization);

        if (eventLocation.getEventName() != null) {
            elocationRoomText.setText(eventLocation.getEventName());
        } else {
            removeView(elocationRoomText, elocationRoom);
        }

        if (eventLocation.getEventBuilding() != null) {
            elocationBuildingText.setText(eventLocation.getEventBuilding());
        } else {
            removeView(elocationBuildingText, elocationBuilding);
        }

        if (eventLocation.getEventOrganization() != null) {
            elocationOrganizationText.setText(eventLocation.getEventOrganization());
        } else {
            removeView(elocationOrganizationText, elocationOrganization);
        }

        if (eventLocation.getFeatureId() != null) {
            focusOnFeature(eventLocation.getFeatureId());
        }
    }

    private void addFocusOnFeature(String featureId) {
        removeFocusOnLastFeature();

        lastFocussedFeature = featureId;
        context.getDeepMap().highlightFeatureWithId(Long.valueOf(featureId));
    }

    @Feature("Routenfindung")
    private void addInformation(ISearchable infoObject) {
        if (infoObject instanceof Person) {
            addPersonInformation((Person) infoObject);
        }

        if (infoObject instanceof EventLocation) {
            addEventLocationInfo((EventLocation) infoObject);
            removeTwitterTab();
        }

        if (infoObject instanceof Office) {
            addOfficeInfo((Office) infoObject);
            removeTwitterTab();
        }

        if (infoObject instanceof Toilet) {
            addToiletInfo((Toilet) infoObject);
            removeTwitterTab();
        }

        if (infoObject instanceof Building) {
            addBuildingInfo((Building) infoObject);
            removeTwitterTab();
        }
        
        if (infoObject instanceof Station){
        	context.opnvManager.getStationByFeature(((Station) infoObject).getFeatureId(), new QueryCallback<Station>() {
                @Override
                public void onFailure(Exception x) {
                }

                @Override
                public void onSuccess(final Station s) {
                    context.opnvManager.queryJourney(s.getHafasID(), new Date(), new ArrayList<String>(), false,
                            new QueryCallback<RnvJourney>() {
                                @Override
                                public void onFailure(Exception x) {
                                }

                                @Override
                                public void onSuccess(final RnvJourney j) {
                                    context.runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            setInfo(s, j);
                                        }
                                    });
                                }
                            });
                }
            });
        }
    }

    @Feature("Routenfindung")
    private void addOfficeInfo(Office office) {
        setLayoutForSubContainer(R.layout.layout_info_office);

        final TextView titleOffice = (TextView) context.findViewById(R.id.office_title_label);
        final TextView buildingOffice = (TextView) context.findViewById(R.id.office_building_label);
        final TextView personOffice = (TextView) context.findViewById(R.id.office_person_label);
        final TextView organizationOffice = (TextView) context.findViewById(R.id.office_organization_label);

        final TextView titleOfficeText = (TextView) context.findViewById(R.id.office_title);
        final TextView buildingOfficeText = (TextView) context.findViewById(R.id.office_building);
        final TextView organizationOfficeText = (TextView) context.findViewById(R.id.office_organization);
        final LinearLayout personsInOffice = (LinearLayout) context.findViewById(R.id.office_persons);

        if (office.getOfficeRoom() != null) {
            titleOfficeText.setText(office.getOfficeRoom());
        } else {
            removeView(titleOfficeText, titleOffice);
        }

        if (office.getOfficeBuilding() != null) {
            buildingOfficeText.setText(office.getOfficeBuilding());
        } else {
            removeView(buildingOfficeText, buildingOffice);
        }

        if (!office.getPersons().isEmpty()) {
            final InfoBoxPersonManager personManager = new InfoBoxPersonManager(context, this);
            final List<TextView> personsViews = personManager.getViewsForPersonsInOffice(office);
            for (final TextView person : personsViews) {
                personsInOffice.addView(person);
            }
        } else {
            removeView(personOffice, personsInOffice);
        }

        if (office.getOfficeOrganization() != null) {
            organizationOfficeText.setText(office.getOfficeOrganization().getOrganizerName());
        } else {
            removeView(organizationOfficeText, organizationOffice);
        }

        if (office.getFeatureId() != null) {
            focusOnFeature(office.getFeatureId());
        }
    }

    @Feature("Routenfindung")
    private void addPersonInformation(Person selectedPerson) {
        setLayoutForSubContainer(R.layout.layout_info_person);

        final TextView titlePerson = (TextView) context.findViewById(R.id.person_title);
        final TextView namePerson = (TextView) context.findViewById(R.id.person_name);
        final TextView occupationPerson = (TextView) context.findViewById(R.id.person_occupation);
        final TextView officePerson = (TextView) context.findViewById(R.id.person_office);

        listView = (ListView) context.findViewById(R.id.tweets);

        if (selectedPerson.getTitle() != null) {
            titlePerson.setText(selectedPerson.getTitle());
        }
        if (selectedPerson.getSurname() != null && selectedPerson.getForename() != null) {
            namePerson.setText(selectedPerson.getForename() + " " + selectedPerson.getSurname());
        } else if (selectedPerson.getSurname() != null && selectedPerson.getForename() == null) {
            namePerson.setText(selectedPerson.getSurname());
        } else if (selectedPerson.getSurname() == null && selectedPerson.getForename() != null) {
            namePerson.setText(selectedPerson.getForename());
        }
        if (selectedPerson.getOccupation() != null) {
            occupationPerson.setText(selectedPerson.getOccupation().getName());
        }
        if (selectedPerson.getRoom().getName() != null) {
            officePerson.setText(selectedPerson.getRoom().getName());
        }

        if (selectedPerson.getRoom().getFeatureId() != null) {
            focusOnFeature(selectedPerson.getRoom().getFeatureId());
        }

        new AsyncRetrieveTweetsTask(this).execute(selectedPerson.getTwitterUserName());
    }

    @Feature("Routenfindung")
    private void addToiletInfo(Toilet toilet) {
        setLayoutForSubContainer(R.layout.layout_info_toilet);

        final TextView titleToilet = (TextView) context.findViewById(R.id.toilet_title_label);
        final TextView nameToilet = (TextView) context.findViewById(R.id.toilet_name_label);
        final TextView buildingToilet = (TextView) context.findViewById(R.id.toilet_building_label);

        final TextView titleToiletText = (TextView) context.findViewById(R.id.toilet_title);
        final TextView nameToiletText = (TextView) context.findViewById(R.id.toilet_name);
        final TextView buildingToiletText = (TextView) context.findViewById(R.id.toilet_building);

        if (toilet.getType() != null) {
            titleToiletText.setText(toilet.getType().getName());
        } else {
            removeView(titleToilet, titleToiletText);
        }

        if (toilet.getToiletName() != null) {
            nameToiletText.setText(toilet.getToiletName());
        } else {
            removeView(nameToiletText, nameToilet);
        }

        if (toilet.getBuildingName() != null) {
            buildingToiletText.setText(toilet.getBuildingName());
        } else {
            removeView(buildingToiletText, buildingToilet);
        }

        if (toilet.getFeatureId() != null) {
            focusOnFeature(toilet.getFeatureId());
        }
    }

    @Feature("opnv")
    public void addInformation(Station station, RnvJourney rnvJourney) {
        ((TextView) context.findViewById(R.id.longname)).setText(station.getLongName());
        ((TextView) context.findViewById(R.id.shortname)).setText(station.getShortName());
        ((TextView) context.findViewById(R.id.ticker)).setText(rnvJourney.getTicker());
        final ListView departureListView = (ListView) context.findViewById(R.id.departure_list);
        final RnvDepartureArrayAdapter adapter = new RnvDepartureArrayAdapter(context, rnvJourney.getListOfDepartures());
        departureListView.setAdapter(adapter);
        focusOnFeature(station.getFeatureId());
    }

    @Feature("Routenfindung")
    private void focusOnFeature(String id) {
        addFocusOnFeature(id);
        context.getDeepMap().moveToFeature(Long.parseLong(id), true);
        final String level = context.getDeepMap().getFeatureWithId(Long.parseLong(id)).getAttributes().get("level");
        context.getDeepMap().setLevel(Integer.parseInt(level));
        context.getElevatorManager().getCurrentFloor().setText(level);
    }

    public void hideInfoBox() {
        layout.setPanelState(PanelState.COLLAPSED);
        layout.invalidate();
    }

    private boolean isMenuInBox() {
        return context.findViewById(R.id.menu_container_ref) != null;
    }

    private void openInfoBox() {
        layout.setPanelState(PanelState.ANCHORED);
    }

    private void removeFocusOnLastFeature() {
        if (lastFocussedFeature != null) {
            context.getDeepMap().unhighlightFeatureWithId(Long.valueOf(lastFocussedFeature));
        }
    }

    private void removeTwitterTab() {
        // Tempory workaround
        final TabHost tabHost = (TabHost) context.findViewById(R.id.tabHost);
        tabHost.getTabWidget().getChildAt(1).setVisibility(View.GONE);
    }

    private void removeView(TextView title, View text) {
        final View titleParent = (View) title.getParent();
        ((ViewGroup) titleParent).removeView(title);
        final View textParent = (View) text.getParent();
        ((ViewGroup) textParent).removeView(text);
    }

    public void resetMenu() {
        setMenuInBox();
        final MapView deepmap = context.getDeepMap();
        deepmap.getAnnotations();
    }

    @Feature("filtern")
    private void setFilterButtonListeners(View menuContainer) {
        final ToggleButton buildingsBtn = menuContainer.findViewById(R.id.menu_button_building);
        final ToggleButton personsBtn = menuContainer.findViewById(R.id.menu_button_person);
        final ToggleButton roomsBtn = menuContainer.findViewById(R.id.menu_button_free_rooms);
        final ToggleButton stationsBtn = menuContainer.findViewById(R.id.menu_button_stations);

        /* Set listener for buildings */
        buildingsBtn.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    uncheckAllButtons(Arrays.asList(personsBtn, roomsBtn, stationsBtn));
                    filterManager.markAllBuildings();
                } else {
                    filterManager.unmarkAll();
                }
            }
        });

        setOnCheckedChangeListenerForPersonsBtn(buildingsBtn, personsBtn, roomsBtn, stationsBtn);

        setOnCheckedChangeListenerForRoomsBtn(buildingsBtn, personsBtn, roomsBtn, stationsBtn);

        setOnCheckedChangeListenerForStationsBtn(buildingsBtn, personsBtn, roomsBtn, stationsBtn);
    }

    private void setOnCheckedChangeListenerForPersonsBtn(final ToggleButton buildingsBtn, final ToggleButton personsBtn,
            final ToggleButton roomsBtn, final ToggleButton stationsBtn) {
        /* Set listener for persons */
        personsBtn.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    uncheckAllButtons(Arrays.asList(buildingsBtn, roomsBtn, stationsBtn));
                    filterManager.markAllPersons();
                } else {
                    filterManager.unmarkAll();
                }
            }
        });
    }

    private void setOnCheckedChangeListenerForRoomsBtn(final ToggleButton buildingsBtn, final ToggleButton personsBtn,
            final ToggleButton roomsBtn, final ToggleButton stationsBtn) {
        /* Set listener for rooms */
        roomsBtn.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    uncheckAllButtons(Arrays.asList(personsBtn, buildingsBtn, stationsBtn));
                    filterManager.markAllRooms();
                } else {
                    filterManager.unmarkAll();
                }
            }
        });
    }

    private void setOnCheckedChangeListenerForStationsBtn(final ToggleButton buildingsBtn,
            final ToggleButton personsBtn, final ToggleButton roomsBtn, final ToggleButton stationsBtn) {
        /* Set listener for stations */
        stationsBtn.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    uncheckAllButtons(Arrays.asList(personsBtn, roomsBtn, buildingsBtn));
                    filterManager.markAllStations();
                } else {
                    filterManager.unmarkAll();
                    filterManager.removeAnnotations();
                }
            }
        });
    }

    @Feature("Routenfindung")
    public void setInfo(ISearchable infoObject) {
        if (state != State.INFO) {
            menuContainer = context.findViewById(R.id.menu_container_ref);
            contentContainer = (LinearLayout) menuContainer.getParent();
            contentContainer.removeView(menuContainer);
            final View infoBoxContainer = context.getLayoutInflater().inflate(R.layout.layout_info_container,
                    contentContainer, false);
            contentContainer.addView(infoBoxContainer);
            addInformation(infoObject);
            openInfoBox();
        } else {
            final View currentInfoBoxContainer = context.findViewById(R.id.info_container);
            if (currentInfoBoxContainer != null) {
                contentContainer = (LinearLayout) currentInfoBoxContainer.getParent();
                contentContainer.removeView(currentInfoBoxContainer);
                final View infoBoxContainer = context.getLayoutInflater().inflate(R.layout.layout_info_container,
                        contentContainer, false);
                contentContainer.addView(infoBoxContainer);
            }
            addInformation(infoObject);
        }
        context.setOnRouteButton(infoObject);
        state = State.INFO;
    }

    @Feature("opnv")
    public void setInfo(Station station, RnvJourney rnvJourney) {
        if (state != State.OPNV) {
            menuContainer = context.findViewById(R.id.menu_container_ref);
            if(menuContainer == null) {
            	menuContainer = context.findViewById(R.id.info_container);
            }
            contentContainer = (LinearLayout) menuContainer.getParent();
            contentContainer.removeView(menuContainer);
            opnvBoxContainer = context.getLayoutInflater().inflate(R.layout.layout_departure_container,
                    contentContainer, false);
            contentContainer.addView(opnvBoxContainer);
            addInformation(station, rnvJourney);
            layout.setPanelState(PanelState.ANCHORED);
            layout.addPanelSlideListener(new PanelSlideListener() {
                @Override
                public void onPanelSlide(View arg0, float arg1) {
                }

                @Override
                public void onPanelStateChanged(View arg0, PanelState arg1, PanelState arg2) {
                    if (PanelState.COLLAPSED.equals(arg2)) {
                        resetMenu();
                    }
                }
            });
        } else {
            addInformation(station, rnvJourney);
        }
        state = State.OPNV;
    }

    private void setLayoutForSubContainer(int subLayoutId) {
        final LinearLayout currentInfoBoxSubContainer = context.findViewById(R.id.info_subcontainer);
        if (currentInfoBoxSubContainer != null) {
            currentInfoBoxSubContainer.removeAllViews();
            final View container = context.getLayoutInflater().inflate(subLayoutId, contentContainer, false);
            currentInfoBoxSubContainer.addView(container);
        }
        setupTabs();
    }

    public void setMenuInBox() {
        LinearLayout container;
        if (!isMenuInBox()) {
            final LinearLayout infoContainer = context.findViewById(R.id.info_container);
            container = infoContainer;
        } else {
            final LinearLayout rnvContainer = context.findViewById(R.id.menu_container_ref);
            container = rnvContainer;
        }
        final LinearLayout contentContainer = (LinearLayout) container.getParent();
        contentContainer.removeView(container);
        final View menuContainer = context.getLayoutInflater().inflate(R.layout.layout_menu_container, contentContainer,
                false);
        contentContainer.addView(menuContainer);
        final String filter = this.context.getFilter();
        if (filter == "building") {
            final ToggleButton buildingsBtn = menuContainer.findViewById(R.id.menu_button_building);
            buildingsBtn.setChecked(true);
        } else if (filter == "room") {
            final ToggleButton roomsBtn = menuContainer.findViewById(R.id.menu_button_free_rooms);
            roomsBtn.setChecked(true);
        } else if (filter == "person") {
            final ToggleButton personsBtn = menuContainer.findViewById(R.id.menu_button_person);
            personsBtn.setChecked(true);
        } else if (filter == "station") {
            final ToggleButton stationBtn = menuContainer.findViewById(R.id.menu_button_stations);
            stationBtn.setChecked(true);
        }
        setFilterButtonListeners(menuContainer);

        state = State.MENU;
    }

    private void setupTabs() {
        final TabHost host = (TabHost) context.findViewById(R.id.tabHost);
        host.setup();

        TabHost.TabSpec spec = host.newTabSpec("Info");
        spec.setContent(R.id.tab1);
        spec.setIndicator("Info");
        host.addTab(spec);

        spec = host.newTabSpec("Twitter");
        spec.setContent(R.id.tab2);
        spec.setIndicator("Twitter");
        host.addTab(spec);
    }

    @Override
    public void tweetSearchProducedResults(List<Status> tweets) {
        View empty = context.findViewById(R.id.empty);
        empty.setVisibility(View.GONE);
    	
    	ArrayList<String> items = new ArrayList<String>();
        if (tweets != null && tweets.isEmpty()) {
            empty.setVisibility(View.VISIBLE);
            listView.setEmptyView(empty);
            return;
        }

        for (int i = 0; i < tweets.size() && i < NUMBER_OF_TWEETS; i++) {
            if (tweets.get(i) != null) {
                items.add(tweets.get(i).getText());
            }
        }
        String[] itemsArr = new String[items.size()];
        itemsArr = items.toArray(itemsArr);

        final ArrayAdapter<String> adapter = new ArrayAdapter<String>(context, android.R.layout.simple_list_item_1,
                itemsArr) {
            @Override
            public View getView(int position, View convertView, ViewGroup parent) {
                final View view = super.getView(position, convertView, parent);
                final TextView tv = (TextView) view.findViewById(android.R.id.text1);
                tv.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 16);
                return view;
            }
        };

        listView.setAdapter(adapter);
    }

    @Feature("filtern")
    private void uncheckAllButtons(List<ToggleButton> btns) {
        if (btns == null) {
            return;
        }
        for (final ToggleButton toggleButton : btns) {
            toggleButton.setChecked(false);
        }
    }
}
