package de.uni_heidelberg.ise.studentnavigator;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Color;
import android.text.SpannableString;
import android.text.style.UnderlineSpan;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.LinearLayout;
import android.widget.TextView;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Person;

@Feature("Routenfindung")
public class InfoBoxPersonManager {
    private class ShowPersonInfoOnTouchListener implements OnTouchListener {
        private final Person person;

        public ShowPersonInfoOnTouchListener(Person person) {
            this.person = person;
        }

        @Override
        public boolean onTouch(View v, MotionEvent e) {
            infoBoxManager.setInfo(person);
            return false;
        }
    }
    private final MainActivity context;

    private final InfoBoxManager infoBoxManager;

    public InfoBoxPersonManager(MainActivity context, InfoBoxManager infoBoxManager) {
        this.context = context;
        this.infoBoxManager = infoBoxManager;
    }

    private String createDisplayText(Person p) {
        return p.getTitle() + " " + p.getForename() + " " + p.getSurname();
    }

    private TextView getTextViewForPerson(Person p) {
        final TextView view = new TextView(context);
        final SpannableString content = new SpannableString(createDisplayText(p));
        content.setSpan(new UnderlineSpan(), 0, content.length(), 0);
        view.setText(content);
        view.setTextColor(Color.BLUE);
        view.setOnTouchListener(new ShowPersonInfoOnTouchListener(p));
        return view;
    }

    public List<TextView> getViewsForPersonsInOffice(Office o) {
        final List<Person> persons = o.getPersons();
        final ArrayList<TextView> personViews = new ArrayList<TextView>();
        for (final Person person : persons)
            personViews.add(getTextViewForPerson(person));
        return personViews;
    }

    public void removeOfficeInfo() {
        final TextView titleOffice = (TextView) context.findViewById(R.id.office_title_label),
                buildingOffice = (TextView) context.findViewById(R.id.office_building_label),
                personOffice = (TextView) context.findViewById(R.id.office_person_label),
                organizationOffice = (TextView) context.findViewById(R.id.office_organization_label),
                titleOfficeText = (TextView) context.findViewById(R.id.office_title),
                buildingOfficeText = (TextView) context.findViewById(R.id.office_building),
                organizationOfficeText = (TextView) context.findViewById(R.id.office_organization);
        final LinearLayout personsInOffice = (LinearLayout) context.findViewById(R.id.office_persons);
        titleOffice.setVisibility(View.GONE);
        buildingOffice.setVisibility(View.GONE);
        personOffice.setVisibility(View.GONE);
        organizationOffice.setVisibility(View.GONE);
        titleOfficeText.setVisibility(View.GONE);
        buildingOfficeText.setVisibility(View.GONE);
        organizationOfficeText.setVisibility(View.GONE);
        personsInOffice.setVisibility(View.GONE);
    }
}
