package de.uni_heidelberg.ise.studentnavigator;

import java.io.BufferedInputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.hdm_i.dm.android.mapsdk.Annotation;
import com.hdm_i.dm.android.mapsdk.HDMFeature;
import com.hdm_i.dm.android.mapsdk.HDMMapCamera;
import com.hdm_i.dm.android.mapsdk.HDMVec3;
import com.hdm_i.dm.android.mapsdk.MapFragment;
import com.hdm_i.dm.android.mapsdk.MapView;
import com.hdm_i.dm.android.mapsdk.MapView.TapDelegate;
import com.j256.ormlite.android.apptools.OpenHelperManager;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.view.ContextThemeWrapper;
import android.view.Menu;
import android.view.MenuItem;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.extern.OpnvManager;
import de.uni_heidelberg.ise.studentnavigator.extern.QueryCallback;
import de.uni_heidelberg.ise.studentnavigator.extern.RnvJourney;
import de.uni_heidelberg.ise.studentnavigator.model.DataToFeatureIdMapper;
import de.uni_heidelberg.ise.studentnavigator.model.DatabaseHelper;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.api.DataManager;
import de.uni_heidelberg.ise.studentnavigator.model.api.IDataManager;
import de.uni_heidelberg.ise.studentnavigator.model.api.ISearchable;

@Feature({"Routenfindung", "filtern", "opnv", "Twitter"})
public class MainActivity extends Activity implements TapDelegate {
    protected static final String PREFERENCE_FIRST_RUN = null;

    private HDMVec3 currentVector;

    private DatabaseHelper databaseHelper;
    private MapView deepMap;
    private ElevatorManager elevatorManager;
    private String filter;
    private InfoBoxManager infoBoxManager;
    private HDMMapCamera initialPerspective;

    OpnvManager opnvManager;
    private SearchBarManager searchBarManager;
    private SearchManager searchManager;

    public DatabaseHelper getDataBaseHelper() {
        if (databaseHelper == null)
            databaseHelper = OpenHelperManager.getHelper(this, DatabaseHelper.class);

        return databaseHelper;
    }

    public MapView getDeepMap() {
        return this.deepMap;
    }

    public ElevatorManager getElevatorManager() {
        return elevatorManager;
    }

    public String getFilter() {
        return filter;
    }

    public InfoBoxManager getInfoBoxManager() {
        return infoBoxManager;
    }

    public HDMMapCamera getInitialPerspective() {
        return initialPerspective;
    }

    private void markAllMapObjects(MapView v) {
        final IDataManager dataManager = new DataManager(getDataBaseHelper().getConnectionSource());
        final List<Office> offices = dataManager.getOffices();
        for (final Office office : offices)
            v.setFeatureAttribute("isOffice", "yes", Long.valueOf(office.getFeatureId()));

        final List<Toilet> toilets = dataManager.getToilets();

        for (final Toilet toilet : toilets)
            v.setFeatureAttribute("isToilet", "yes", Long.valueOf(toilet.getFeatureId()));

        for (final EventLocation eventLocation : dataManager.getEventLocations())
            v.setFeatureAttribute("isEventLocation", "yes", Long.valueOf(eventLocation.getFeatureId()));
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setFilter("none");
        setContentView(R.layout.activity_main);
        infoBoxManager = new InfoBoxManager(this);
        searchBarManager = new SearchBarManager(this, infoBoxManager);
        elevatorManager = new ElevatorManager(this);
        searchManager = new SearchManager(this);
        opnvManager = new OpnvManager(new DataManager(getDataBaseHelper().getConnectionSource()));

        final MapFragment mapFragment = (MapFragment) getFragmentManager().findFragmentById(R.id.deepmap_fragment);
        final MainActivity self = this;
        mapFragment.loadMap(new MapView.ReadyDelegate() {

            @Override
            public void onReady(MapView v) {
                deepMap = v;
                new LocationHandler(MainActivity.this);
                final int maximumLevel = deepMap.getMaximumLevel();
                deepMap.setLevel(maximumLevel);
                elevatorManager.getCurrentFloor().setText(String.valueOf(maximumLevel));
                // set the tapdelegate
                v.setTapDelegate(self);

                // save initial perspective of map
                initialPerspective = v.getClonedCamera();
                final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(self);
                final boolean firstRun = p.getBoolean(PREFERENCE_FIRST_RUN, true);
                p.edit().putBoolean(PREFERENCE_FIRST_RUN, false).commit();
                if (firstRun) {
                    final BufferedInputStream featureIdsStream = new BufferedInputStream(
                            getResources().openRawResource(R.raw.feature_ids)),
                            dataFileStream = new BufferedInputStream(
                                    getResources().openRawResource(R.raw.mathematikon));
                    new DataToFeatureIdMapper(deepMap, featureIdsStream, dataFileStream,
                            new DataManager(self.getDataBaseHelper().getConnectionSource()));
                }
                markAllMapObjects(v);
            }
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu m) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, m);
        return true;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (databaseHelper == null)
            return;
        OpenHelperManager.releaseHelper();
        databaseHelper = null;
    }

    @Override
    public void onLongPress(List<HDMFeature> arg0, final HDMVec3 arg1) {
        final CharSequence choices[] = new CharSequence[] { StringConstants.setAsStart, StringConstants.setAsDestination };
        //ContextThemeWrapper ctw = new ContextThemeWrapper(this, R.style.AppTheme_Alert);
        final AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(StringConstants.usage);
        currentVector = arg1;
        builder.setItems(choices, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface i, int which) {
                // the user clicked on point[which]
                if (which == 0) {
                    if (searchBarManager.getStartAnnotation() != null)
                        getDeepMap().removeAnnotation(searchBarManager.getStartAnnotation());

                    final Annotation annotation = new Annotation(StringConstants.start, arg1.getX(), arg1.getY(), arg1.getZ());
                    searchBarManager.setSearchView(StringConstants.startInBrackets);
                    searchBarManager.setStartVector(currentVector);
                    searchBarManager.setStartAnnotation(annotation);
                    getDeepMap().addAnnotation(annotation);
                }
                if (which != 1)
                    return;
                if (searchBarManager.getDestinationAnnotation() != null)
                    getDeepMap().removeAnnotation(searchBarManager.getDestinationAnnotation());
                final Annotation annotation = new Annotation(StringConstants.destination, arg1.getX(), arg1.getY(), arg1.getZ());
                searchBarManager.setSearchViewDestination(StringConstants.destinationInBrackets);
                searchBarManager.setDestinationVector(currentVector);
                searchBarManager.setDestinationAnnotation(annotation);
                getDeepMap().addAnnotation(annotation);
            }
        });
        builder.show();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem i) {
        return i.getItemId() == R.id.action_settings || super.onOptionsItemSelected(i);
    }

    @Override
    @Feature("opnv")
    public void onSingleTap(List<HDMFeature> fs, HDMVec3 arg1) {
        if (fs.isEmpty())
            return;
        final String fId = ((Long) fs.get(0).getFeatureId()).toString();
        if (!opnvManager.isStation(((Long) fs.get(0).getFeatureId()).toString())) {
            if (searchManager.searchByFid(fId) != null)
                infoBoxManager.setInfo(searchManager.searchByFid(fId));
        } else {
            final MainActivity self = this;
            opnvManager.getStationByFeature(fId, new QueryCallback<Station>() {
                @Override
                public void onFailure(Exception x) {
                }

                @Override
                public void onSuccess(final Station s) {
                    opnvManager.queryJourney(s.getHafasID(), new Date(), new ArrayList<String>(), false,
                            new QueryCallback<RnvJourney>() {
                                @Override
                                public void onFailure(Exception x) {
                                }

                                @Override
                                public void onSuccess(final RnvJourney j) {
                                    self.runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            self.infoBoxManager.setInfo(s, j);
                                        }
                                    });
                                }
                            });
                }
            });
        }
    }

    public void setFilter(String filter) {
        this.filter = filter;
    }
    
    public void setOnRouteButton(ISearchable searchable){
    	searchBarManager.setInfoboxDestinationLocation(searchable);
    	searchBarManager.setInfoNavigationButton();
    }
}