package de.uni_heidelberg.ise.studentnavigator;

import java.util.List;

import com.hdm_i.dm.android.mapsdk.Annotation;
import com.hdm_i.dm.android.mapsdk.HDMRoutingPathFeature;
import com.hdm_i.dm.android.mapsdk.HDMVec3;
import com.hdm_i.dm.android.mapsdk.MapView;

import android.app.Activity;
import android.text.TextUtils;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.SearchView;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.MapObject;
import de.uni_heidelberg.ise.studentnavigator.model.api.ISearchable;


@Feature("Routenfindung")
public class SearchBarManager {

    private static final int MAXIMUM_SEARCH_RESULTS = 5;
    private static final int MIN_QUERY_LENGTH = 2;
    private final Activity context;
    private Annotation destinationAnnotation;

    private MapObject destinationLocation;
    private HDMVec3 destinationVector;
    private final InfoBoxManager infoBoxHandler;
    private SearchManager manager;
    private final LinearLayout navigationContainer, searchContainer;
    private NavigationInfoManger navigationInfoManager;

    private HDMRoutingPathFeature route;
    private final ImageButton routeButton, routeCloseButton, stopNavigationButton;
    private ImageButton infoNavigationButton;
    private final RelativeLayout routeButtonLayout;
    private final SearchView searchView;
    private final SearchView searchViewDestination;

    private Annotation startAnnotation;
    private MapObject startLocation;
    private HDMVec3 startVector;
    
    private ISearchable infoboxSearchable;

    private final ListView suggestionList;

    public SearchBarManager(Activity context, InfoBoxManager infoBoxHandler) {
        this.context = context;
        this.infoBoxHandler = infoBoxHandler;
        this.searchView = (SearchView) context.findViewById(R.id.search);
        this.searchViewDestination = (SearchView) context.findViewById(R.id.search2);
        this.routeButton = (ImageButton) context.findViewById(R.id.routeButton);
        this.routeButtonLayout = (RelativeLayout) context.findViewById(R.id.routeButtonLayout);
        this.routeCloseButton = (ImageButton) context.findViewById(R.id.routeCloseButton);
        this.stopNavigationButton = (ImageButton) context.findViewById(R.id.stop_navigation_button);
        this.searchContainer = (LinearLayout) context.findViewById(R.id.searchContainer);
        this.suggestionList = (ListView) context.findViewById(R.id.search_suggestions);
        this.navigationContainer = (LinearLayout) context.findViewById(R.id.navigation_info);

        initSearchManager();
        initialize();
    }

    @Feature("Routenfindung")
    private boolean doOnQueryTextChange(String query, SearchView v) {
        if (TextUtils.isEmpty(query) && routeCloseButton.getVisibility() == View.GONE)
            showRouteButton();
        if (minQueryLengthReached(query))
            showSearchResults(v);
        else
            suggestionList.setVisibility(View.GONE);
        return false;
    }

    public Annotation getDestinationAnnotation() {
        return destinationAnnotation;
    }

    public HDMVec3 getDestinationVector() {
        return destinationVector;
    }

    public Annotation getStartAnnotation() {
        return startAnnotation;
    }

    public HDMVec3 getStartVector() {
        return startVector;
    }

    private void initialize() {
        setOnQueryTextListener();
        setOnRouteButtonClickListener();
        setStopNavigationButtonClickListener();

        navigationInfoManager = new NavigationInfoManger(context);
    }

    private void initSearchManager() {
        this.manager = new SearchManager(context);
    }

    private boolean minQueryLengthReached(String query) {
        return query.length() > MIN_QUERY_LENGTH;
    }

    private void removeAnnotation(Annotation a) {
        if (a != null)
            ((MainActivity) context).getDeepMap().removeAnnotation(a);

    }

    private void removeRoute() {
        ((MainActivity) context).getDeepMap().removeFeature(route);
    }

    public void setDestinationAnnotation(Annotation a) {
        destinationAnnotation = a;
    }

    public void setDestinationVector(HDMVec3 v) {
        final MapView mapView = ((MainActivity) context).getDeepMap();
        destinationVector = v;

        if (searchViewDestination.getVisibility() == View.GONE) {
            searchViewDestination.setVisibility(View.VISIBLE);
            showRouteCloseButton();
        }

        if (startLocation != null) {
            showRouteWithVector(mapView.getCoordinateFromFeatureWithId(Long.parseLong(startLocation.getFeatureId())),
                    destinationVector);
            startNavigationMode();
        }

        if (startVector == null)
            return;
        showRouteWithVector(startVector, destinationVector);
        startNavigationMode();
    }

    private void setOnQueryTextListener() {
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextChange(String query) {
                if (query.startsWith("[") && !query.endsWith("]")) {
                    searchView.setQuery("", false);
                    ((MainActivity) context).getDeepMap();
                    removeAnnotation(startAnnotation);
                    startAnnotation = null;
                    startVector = null;
                }
                if (!TextUtils.isEmpty(query))
                    return SearchBarManager.this.doOnQueryTextChange(query, searchView);
                if (searchViewDestination.getVisibility() == View.VISIBLE)
                    searchView.setQueryHint(StringConstants.currentPosition);
                else {
                    searchView.setQueryHint(StringConstants.enterSearch);
                    startLocation = null;
                }
                infoBoxHandler.hideInfoBox();
                return SearchBarManager.this.doOnQueryTextChange(query, searchView);

            }

            @Override
            public boolean onQueryTextSubmit(String s) {
                return false;
            }

        });

        searchViewDestination.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextChange(String query) {
                if (query.startsWith("[") && !query.endsWith("]")) {
                    searchViewDestination.setQuery("", false);
                    ((MainActivity) context).getDeepMap();
                    removeAnnotation(destinationAnnotation);
                    destinationAnnotation = null;
                    destinationVector = null;
                }
                if (TextUtils.isEmpty(query))
                    searchViewDestination.setQueryHint(StringConstants.enterDestination);
                infoBoxHandler.hideInfoBox();
                return SearchBarManager.this.doOnQueryTextChange(query, searchViewDestination);
            }

            @Override
            public boolean onQueryTextSubmit(String s) {
                return false;
            }

        });

        final ImageView closeButton = (ImageView) this.searchView.findViewById(searchView.getContext().getResources().getIdentifier("android:id/search_close_btn", null, null));
        closeButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                ((MainActivity) context).getDeepMap();
                removeAnnotation(startAnnotation);
                startAnnotation = null;
                startVector = null;
                startLocation = null;
                searchView.setQuery("", false);
            }
        });

        ((ImageView) this.searchViewDestination.findViewById(searchViewDestination.getContext().getResources()
                .getIdentifier("android:id/search_close_btn", null, null)))
                        .setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                ((MainActivity) context).getDeepMap();
                                removeAnnotation(destinationAnnotation);
                                destinationAnnotation = null;
                                destinationVector = null;
                                destinationLocation = null;
                                searchViewDestination.setQuery("", false);
                            }
                        });

    }

    private void setOnRouteButtonClickListener() {
        routeButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                searchViewDestination.setVisibility(View.VISIBLE);
                searchViewDestination.setIconified(false);
                searchViewDestination.setQuery(searchView.getQuery(), false);
                searchView.setQuery("", false);
                searchView.setQueryHint(StringConstants.currentPosition);
                showRouteCloseButton();
                showSearchResults(searchViewDestination);

            }
        });
        setOnRoutingCloseListener(routeCloseButton);
        setOnStopNavigationListener(stopNavigationButton);
    }

    private void setOnRoutingCloseListener(ImageButton b) {
        b.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                searchViewDestination.setQuery("", false);
                searchViewDestination.setVisibility(View.GONE);
                searchView.setQuery("", false);
                searchView.setQueryHint(StringConstants.enterSearch);
                startLocation = null;
                destinationLocation = null;
                startVector = null;
                destinationVector = null;
                ((MainActivity) context).getDeepMap();
                removeAnnotation(startAnnotation);
                removeAnnotation(destinationAnnotation);
                showRouteButton();
                removeRoute();
            }
        });
    }

    private void setOnStopNavigationListener(ImageButton b) {
        b.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                searchViewDestination.setVisibility(View.GONE);
                searchView.setQuery("", false);
                showRouteButton();
            }
        });
    }

    protected void setSearchView(CharSequence query) {
        searchView.setQuery(query, false);
    }

    protected void setSearchViewDestination(CharSequence query) {
        searchViewDestination.setQuery(query, false);
    }

    public void setStartAnnotation(Annotation a) {
        startAnnotation = a;
    }

    public void setStartVector(HDMVec3 v) {
        final MapView mapView = ((MainActivity) context).getDeepMap();
        startVector = v;

        if (searchViewDestination.getVisibility() == View.GONE) {
            searchViewDestination.setVisibility(View.VISIBLE);
            showRouteCloseButton();
        }

        if (destinationLocation != null) {
            showRouteWithVector(startVector,
                    mapView.getCoordinateFromFeatureWithId(Long.parseLong(destinationLocation.getFeatureId())));
            startNavigationMode();
        }
        if (destinationVector == null)
            return;
        showRouteWithVector(startVector, destinationVector);
        startNavigationMode();
    }

    private void setStopNavigationButtonClickListener() {
        stopNavigationButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                stopNavigationMode();
            }
        });
    }

    @Feature("Routenfindung")
    private void showRoute(long start, long destination) {
        final MapView mapView = ((MainActivity) context).getDeepMap();

        showRouteWithVector(mapView.getCoordinateFromFeatureWithId(start),
                mapView.getCoordinateFromFeatureWithId(destination));
    }

    private void showRouteButton() {
        routeButtonLayout.setVisibility(View.VISIBLE);
        routeButton.setVisibility(View.VISIBLE);
        routeCloseButton.setVisibility(View.GONE);
    }

    private void showRouteCloseButton() {
        routeButtonLayout.setVisibility(View.VISIBLE);
        routeButton.setVisibility(View.GONE);

        routeCloseButton.setVisibility(View.VISIBLE);
    }

    private void showRouteWithVector(HDMVec3 startVector, HDMVec3 destinationVector) {
        final MainActivity mainActivity = (MainActivity) context;
        route = mainActivity.getDeepMap().CalculateRouteBetweenPoints(startVector, destinationVector);
        mainActivity.getDeepMap().navigateWithPath(route, MapView.HDMUserTrackingMode.HDMUserTrackingModeNone);
    }

    @Feature("Routenfindung")
    public void showSearchResults(final SearchView searchedView) {
        if (!minQueryLengthReached(searchedView.getQuery().toString()))
            return;

        suggestionList.setVisibility(View.VISIBLE);
        final List<ISearchable> suggestions = manager.searchFor(searchedView.getQuery().toString(), ((MainActivity) context).getFilter());
        if (suggestions.size() > MAXIMUM_SEARCH_RESULTS)
            suggestions.subList(0, MAXIMUM_SEARCH_RESULTS);

        suggestionList.setAdapter(new SearchResultsArrayAdapter(context, suggestions));
        suggestionList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {

                final ISearchable selectedItem = suggestions.get(i);
                searchedView.setQuery(selectedItem.getSearchableTitle(), false);
                searchedView.clearFocus();

                suggestionList.setVisibility(View.GONE);

                if (routeCloseButton.getVisibility() == View.GONE) {
                    routeButtonLayout.setVisibility(View.GONE);
                    routeButtonLayout.invalidate();
                }
                infoBoxHandler.setInfo(selectedItem);
                setInfoNavigationButton();

                if (searchedView.getId() == R.id.search)
                    startLocation = selectedItem.getMapObject();
                else if (searchedView.getId() == R.id.search2)
                    destinationLocation = selectedItem.getMapObject();
                
                final MapView mapView = ((MainActivity) context).getDeepMap();
                
                if (destinationLocation != null && LocationHandler.getCurrentLocation() != null
                        && startLocation == null) {
                    
                    showRouteWithVector(LocationHandler.getCurrentLocationAsVector(),
                            mapView.getCoordinateFromFeatureWithId(Long.parseLong(destinationLocation.getFeatureId())));
                    startNavigationMode();
                }

                if (destinationLocation != null && startLocation != null) {
                  
                    showRoute(Long.parseLong(startLocation.getFeatureId()), Long.parseLong(destinationLocation.getFeatureId()));
                    startNavigationMode();
                }

                if (startLocation != null && destinationVector != null) {
                    
                    showRouteWithVector(
                            mapView.getCoordinateFromFeatureWithId(Long.parseLong(startLocation.getFeatureId())),
                            destinationVector);
                    startNavigationMode();
                }

                if (destinationLocation == null || startVector == null)
                    return;
                showRouteWithVector(startVector,
                        mapView.getCoordinateFromFeatureWithId(Long.parseLong(destinationLocation.getFeatureId())));
                startNavigationMode();

            }
        });

    }
    
    public void setInfoNavigationButton(){
    	infoNavigationButton = (ImageButton) context.findViewById(R.id.info_navigation_button);
        infoNavigationButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                searchViewDestination.setVisibility(View.VISIBLE);
                if(startLocation == null && infoboxSearchable != null){
                	destinationLocation = infoboxSearchable.getMapObject();
                	searchViewDestination.setQuery(infoboxSearchable.getSearchableTitle().toString(), false);
                	infoboxSearchable = null;
                }
                else{
                    searchViewDestination.setQuery(searchView.getQuery(), false);
                	destinationLocation = startLocation;
                }
                searchView.setQuery("", false);
                searchView.setQueryHint(StringConstants.currentPosition);
                if (LocationHandler.getCurrentLocation() == null)
                    searchView.requestFocus();
                else {
                    showRouteWithVector(LocationHandler.getCurrentLocationAsVector(),
                            ((MainActivity) context).getDeepMap().getCoordinateFromFeatureWithId(
                                    Long.parseLong(destinationLocation.getFeatureId())));
                    startNavigationMode();
                }
                showRouteCloseButton();
            }
        });
    }

    public void setInfoboxDestinationLocation(ISearchable infoboxSearchable){
    	this.infoboxSearchable = infoboxSearchable;
    }
    
    private void startNavigationMode() {
        navigationInfoManager.setNavigationContent(route);
        routeButtonLayout.setVisibility(View.INVISIBLE);
        searchContainer.setVisibility(View.INVISIBLE);
        navigationContainer.setVisibility(View.VISIBLE);
    }

    public void stopNavigationMode() {
        routeButtonLayout.setVisibility(View.VISIBLE);
        searchContainer.setVisibility(View.VISIBLE);
        navigationContainer.setVisibility(View.GONE);
        removeRoute();

    }

}
