package de.uni_heidelberg.ise.studentnavigator;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Building;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.Person;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.api.ISearchable;

@Feature("Routenfindung")
public class SearchResultsArrayAdapter extends ArrayAdapter<ISearchable> {
    Context context;

    public SearchResultsArrayAdapter(Context context, List<ISearchable> values) {
        super(context, R.layout.layout_search_result, values);
        this.context = context;
    }

    private int getIconId(ISearchable s) {
        return s instanceof Person ? R.drawable.ic_menu_persons
                : s instanceof Building ? R.drawable.ic_menu_building
                        : !(s instanceof EventLocation) && s instanceof Toilet ? R.drawable.ic_wc : R.drawable.ic_room;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        final View rowView = ((LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE)).inflate(R.layout.layout_search_result, parent, false);
        ((ImageView) rowView.findViewById(R.id.type_pic)).setImageResource(getIconId(getItem(position)));
        ((TextView) rowView.findViewById(R.id.name)).setText(getItem(position).getSearchableTitle());
        ((TextView) rowView.findViewById(R.id.subtitle)).setText(getItem(position).getSearchableDescription());
        return rowView;
    }

    public void onAddItem(Person item) {
        this.add(item);
        notifyDataSetChanged();
    }

    public void onRemove(Person item) {
        remove(item);
        notifyDataSetChanged();
    }
}
