package de.uni_heidelberg.ise.studentnavigator.extern;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import com.google.gson.Gson;
import com.squareup.okhttp.Callback;
import com.squareup.okhttp.HttpUrl;
import com.squareup.okhttp.OkHttpClient;
import com.squareup.okhttp.Request;
import com.squareup.okhttp.Response;

import android.util.Log;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.api.IDataManager;

@Feature("opnv")
public class OpnvManager implements IOpnvManager {
    private final OkHttpClient client = new OkHttpClient();

    private final IDataManager dataManager;

    private final Gson gson = new Gson();
    public OpnvManager(IDataManager dataManager) {
        this.dataManager = dataManager;
    }

    @Override
    public void getStation(String hafasID, QueryCallback<Station> result) {
    }

    @Override
    public void getStationByFeature(String featureID, QueryCallback<Station> result) {
        Station station = null;
        for (final Station s : dataManager.getStations())
            if (Long.parseLong(s.getFeatureId()) == Long.parseLong(featureID)) {
                station = s;
                break;
            }
        if (station != null)
            result.onSuccess(station);
        else
            result.onFailure(new Exception());
    }

    @Override
    public boolean isStation(String featureID) {
        for (final Station station : dataManager.getStations())
            if (Long.parseLong(station.getFeatureId()) == Long.parseLong(featureID))
                return true;
        return false;
    }

    @Override
    public void queryJourney(String hafasID, Date time, List<String> poles, boolean needPlatformDetail,
            final QueryCallback<RnvJourney> result) {
        final Request request = new Request.Builder()
                .url(new HttpUrl.Builder().scheme("http").host("rnv.the-agent-factory.de").port(8080)
                        .addPathSegment("easygo2").addPathSegment("api").addPathSegment("regions").addPathSegment("rnv")
                        .addPathSegment("modules").addPathSegment("stationmonitor").addPathSegment("element")
                        .addQueryParameter("hafasID", hafasID)
                        .addQueryParameter("time", new SimpleDateFormat("yyyy-MM-dd HH:mm").format(time))
                        .addQueryParameter("poles", android.text.TextUtils.join(";", poles))
                        .addQueryParameter("needPlatformDetail", ((Boolean) needPlatformDetail).toString()).build())
                .addHeader("RNV_API_TOKEN", "idq890robgnb87d41af4in90bk").build();
        Log.d("RNV", request.urlString());
        try {
            client.newCall(request).enqueue(new Callback() {

                @Override
                public void onFailure(Request arg0, IOException arg1) {
                    result.onFailure(arg1);
                }

                @Override
                public void onResponse(Response r) throws IOException {
                    if (!r.isSuccessful())
                        result.onFailure(new IOException("Unexpected code " + r));
                    else
                        result.onSuccess(gson.fromJson(r.body().string(), RnvJourney.class));
                }
            });
        }

        catch (final Exception e) {
            Log.d("rnv", e.toString());
        }
    }

    public void queryStation(String hafasID, final QueryCallback<RnvJourney> result) throws Exception {
        final Request request = new Request.Builder()
                .url(new HttpUrl.Builder().scheme("http").host("rnv.the-agent-factory.de").port(8080)
                        .addPathSegment("easygo2").addPathSegment("api").addPathSegment("regions").addPathSegment("rnv")
                        .addPathSegment("modules").addPathSegment("stationmonitor").addPathSegment("element")
                        .addQueryParameter("hafasID", hafasID).build())
                .addHeader("RNV_API_TOKEN", "idq890robgnb87d41af4in90bk").build();
        Log.d("RNV", request.urlString());
        try {
            client.newCall(request).enqueue(new Callback() {
                @Override
                public void onFailure(Request arg0, IOException arg1) {
                    result.onFailure(arg1);
                }

                @Override
                public void onResponse(Response r) throws IOException {
                    if (!r.isSuccessful())
                        result.onFailure(new IOException("Unexpected code " + r));
                    else
                        result.onSuccess(gson.fromJson(r.body().string(), RnvJourney.class));
                }
            });
        } catch (final Exception e) {
            Log.d("rnv", e.toString());
        }
    }
}
