package de.uni_heidelberg.ise.studentnavigator.model;

import java.util.ArrayList;
import java.util.Calendar;

import com.j256.ormlite.support.ConnectionSource;

import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Person.Occupation;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet.Type;
import de.uni_heidelberg.ise.studentnavigator.model.api.DataManager;

@Feature("Routenfindung")
public class DataGenerator {

	private DataManager dataManager;

	public DataGenerator(ConnectionSource connectionSource) {
		this.dataManager = new DataManager(connectionSource);
	}

	public void generate() {

		// ###Buildings###
		// FeatureID|Name im Lageplan //Beschreibung

		Building mathematikonHDMobil = dataManager.addBuilding("MA 5", "Berliner Str. 41, 69120 Heidelberg", "6860");// Mathematikon
																														// HD
																														// Mobil
		SetOpeningHoursWeek(mathematikonHDMobil, 16, 6);

		SetOpeningHoursWeek(
				dataManager.addBuilding("Mathematikon SW", "Berliner Strae 41-49, 69120 Heidelberg", "563"), 12, 7);

		SetOpeningHoursWeek(
				dataManager.addBuilding("Mathematikon NO", "Berliner Strae 41-49, 69120 Heidelberg", "3291"), 12, 7);

		SetOpeningHoursWeek(
				dataManager.addBuilding("Mathematikon REWE", "Berliner Strae 41-49, 69120 Heidelberg", "3773"), 20, 6);

		SetOpeningHoursWeek(dataManager.addBuilding("DKFZ", "Im Neuenheimer Feld 280, 69120 Heidelberg", "833"), 10, 8);

		SetOpeningHoursWeek(dataManager.addBuilding("ZM5.1", "Im Neuenheimer Feld 304, 69120 Heidelberg", "7503"), 10,
				8);

		// Umweltphysik
		SetOpeningHoursWeek(dataManager.addBuilding("Institut fr Umweltphysik",
				"Im Neuenheimer Feld 229, 69120 Heidelberg", "4070"), 12, 7);

		SetOpeningHoursWeek(dataManager.addBuilding("BIO2.1", "Im Neuenheimer Feld 267, 69120 Heidelberg", "7527"), 10,
				8);

		SetOpeningHoursWeek(dataManager.addBuilding("NHF2", "Im Neuenheimer Feld 2, 69120 Heidelberg", "7516"), 10, 8);

		SetOpeningHoursWeek(dataManager.addBuilding("INF 227 Kirchhoff-Institut fr Physik (KIP)",
				"Im Neuenheimer Feld 227, 69120 Heidelberg", "2673"), 16, 6);

		// ###Stations###
		// noch keine FeatureId für eine Haltestelle

		dataManager.addOrganization("Institut fr Informatik");
		Organization heidelbergMobil = dataManager.addOrganization("Heidelberg Mobil"),
				gebauedeverwaltungSchmidt = dataManager.addOrganization("Gebudeverwaltung Schmidt"),
				uniHd = dataManager.addOrganization("Universität Heidelberg");
		dataManager.addEventLocation("Room 228", mathematikonHDMobil, gebauedeverwaltungSchmidt, "4566");
		dataManager.addEventLocation("Room 871", mathematikonHDMobil, uniHd, "19412");
		dataManager.addEventLocation("Room RC56", mathematikonHDMobil, gebauedeverwaltungSchmidt, "20619");
		dataManager.addEventLocation("Room 228", mathematikonHDMobil, gebauedeverwaltungSchmidt, "4566");
		dataManager.addEventLocation("Room 871", mathematikonHDMobil, uniHd, "19412");
		dataManager.addEventLocation("Room 228", mathematikonHDMobil, gebauedeverwaltungSchmidt, "4566");
		EventLocation eventLocationTokio = dataManager.addEventLocation("Room Tokio", mathematikonHDMobil,
				heidelbergMobil, "21572");
		Office office718 = dataManager.addOffice("Room 718", mathematikonHDMobil, heidelbergMobil, "6906"),
				office719 = dataManager.addOffice("Room 719", mathematikonHDMobil, heidelbergMobil, "7275"),
				office320 = dataManager.addOffice("Room 320", mathematikonHDMobil, uniHd, "7647"),
				office318 = dataManager.addOffice("Room 318", mathematikonHDMobil, uniHd, "7652");

		dataManager.addToilet(Type.MALE, "Room 19", mathematikonHDMobil, gebauedeverwaltungSchmidt, "6819");
		dataManager.addToilet(Type.FEMALE, "Room 18", mathematikonHDMobil, gebauedeverwaltungSchmidt, "6820");
		dataManager.addToilet(Type.MALE, "Room 19", mathematikonHDMobil, gebauedeverwaltungSchmidt, "6819");
		dataManager.addToilet(Type.DISABLED, "Room 9", mathematikonHDMobil, gebauedeverwaltungSchmidt, "6829");
		// ###Events###
		// ID|Period|Room|Description|Organizer
		ArrayList<Organizer> organizers = new ArrayList<Organizer>();
		organizers.add(heidelbergMobil);
		Event party = dataManager.addEvent(eventLocationTokio, "Weihnachtsfeier", organizers);

		Calendar partyBegin = Calendar.getInstance();
		partyBegin.set(Calendar.YEAR, 2017);
		partyBegin.set(Calendar.MONTH, Calendar.DECEMBER);
		partyBegin.set(Calendar.DAY_OF_MONTH, 18);
		partyBegin.set(Calendar.HOUR, 20);

		Calendar partyEnd = Calendar.getInstance();
		partyEnd.set(Calendar.YEAR, 2017);
		partyEnd.set(Calendar.MONTH, Calendar.DECEMBER);
		partyEnd.set(Calendar.DAY_OF_MONTH, 19);
		partyEnd.set(Calendar.HOUR, 4);
		dataManager.addPeriod(party, 6, partyBegin.getTime(), partyEnd.getTime(), Period.Repeating.ONCE);
		dataManager.addPerson("Daniel", "Ried", "Herr", "01672410912", "01672410913", "daniel.ried@hdm.de",
				"heidelberg.mobil.de", Occupation.UNI_EXTERNAL, heidelbergMobil, office718, "");
		dataManager.addPerson("Michael", "Gertz", "Herr", "094172214", "094172215", "michael.gertz@uni-heidelberg.de",
				"", Occupation.DOCENT, uniHd, office320, "");
		dataManager.addPerson("Markus", "Mai", "Herr", "01231415", "01231416", "mmai@gmail.com", "mmai.com",
				Occupation.UNI_EXTERNAL, heidelbergMobil, office719, "");
		dataManager.addPerson("Barbara", "Paech", "Frau", "0526182311", "0526182312", "michael.gertz@uni-heidelberg.de",
				"", Occupation.DOCENT, uniHd, office318, "");
		dataManager.addPerson("Elvis", "Presley", "Herr", "", "", "", "", Occupation.UNI_EXTERNAL, uniHd, office318,
				"ElvisPresley");
		dataManager.addStation("1228", ((Long) 8112L).toString(), "Bunsengymnasium", "NHBG");
		dataManager.addStation("1226", ((Long) 8115L).toString(), "Technologiepark", "NHTP");
	}

	private void SetOpeningHoursWeek(Building b, int hours, int startingHour) {

		int day = 2;// 2.January 2017 = monday

		Calendar startDay = Calendar.getInstance();
		startDay.set(Calendar.YEAR, 2017);
		startDay.set(Calendar.MONTH, Calendar.JANUARY);
		startDay.set(Calendar.DAY_OF_MONTH, day);
		startDay.set(Calendar.HOUR, startingHour);

		Calendar endDate = Calendar.getInstance();
		endDate.set(Calendar.YEAR, 9999);
		endDate.set(Calendar.MONTH, Calendar.DECEMBER);
		endDate.set(Calendar.DAY_OF_MONTH, 10);
		endDate.set(Calendar.HOUR, 22);

		for (int i = 0; i < 5; ++i) {
			dataManager.addPeriod(b, hours, startDay.getTime(), endDate.getTime(), Period.Repeating.WEEKLY);
			day += 1;
		}
	}

}
