package de.uni_heidelberg.ise.studentnavigator.model;

import java.sql.SQLException;

import com.j256.ormlite.android.apptools.OrmLiteSqliteOpenHelper;
import com.j256.ormlite.support.ConnectionSource;
import com.j256.ormlite.table.TableUtils;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;
import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.R;

/**
 * Database helper class used to manage the creation and upgrading of your database. This class also usually provides
 * the DAOs used by the other classes.
 */
@Feature("Routenfindung")
public class DatabaseHelper extends OrmLiteSqliteOpenHelper {
    /** Name of the database file for your application -- change to something appropriate for your app. */
    private static final String DATABASE_NAME = "studentNavigator.db";
    /** Any time you make changes to your database objects, you may have to increase the database version. */
    private static final int DATABASE_VERSION = 1;

    public DatabaseHelper(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION, R.raw.ormlite_config);
    }

    /**
     * This is called when the database is first created. Usually you should call createTable statements here to create
     * the tables that will store your data.
     */
    @Override
    public void onCreate(SQLiteDatabase db, ConnectionSource s) {
        try {
            Log.i(DatabaseHelper.class.getName(), "onCreate");
            // Create needed tables
            TableUtils.createTable(s, Period.class);
            TableUtils.createTable(s, Building.class);
            TableUtils.createTable(s, Station.class);
            TableUtils.createTable(s, Office.class);
            TableUtils.createTable(s, Toilet.class);
            TableUtils.createTable(s, EventLocation.class);
            TableUtils.createTable(s, Event.class);
            TableUtils.createTable(s, Organization.class);
            TableUtils.createTable(s, Person.class);
            TableUtils.createTable(s, EventOrganization.class);

            (new DataGenerator(s)).generate();
        } catch (final SQLException e) {
            Log.e(DatabaseHelper.class.getName(), "Can't create database", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * This is called when your application is upgraded and it has a higher version number. This allows you to adjust
     * the various data to match the new version number.
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, ConnectionSource s, int oldVersion, int newVersion) {
        // Drop tables if needed
        /*
         * try { Log.i(DatabaseHelper.class.getName(), "onUpgrade"); TableUtils.dropTable(connectionSource,
         * Contact.class, true);
         * 
         * // after we drop the old databases, we create the new ones onCreate(db, connectionSource); } catch
         * (SQLException e) { Log.e(DatabaseHelper.class.getName(), "Can't drop databases", e); throw new
         * RuntimeException(e); }
         */
    }
}