package de.uni_heidelberg.ise.studentnavigator.model;

import java.util.Date;

import com.j256.ormlite.field.DataType;
import com.j256.ormlite.field.DatabaseField;
import com.j256.ormlite.table.DatabaseTable;

import de.uhd.ifi.se.feature.annotations.java.Feature;

@DatabaseTable(tableName = "periods")
@Feature("Routenfindung")
public class Period {
    public enum Repeating {
        DAILY, MONTHLY, ONCE, WEEKLY
    }

    @DatabaseField(dataType = DataType.DATE_STRING)
    private Date begin;
    @DatabaseField(foreign = true, foreignAutoRefresh = true)
    private Building building;
    @DatabaseField
    private Integer duration;
    @DatabaseField(dataType = DataType.DATE_STRING)
    private Date end;
    @DatabaseField(foreign = true, foreignAutoRefresh = true)
    private Event event;
    @DatabaseField(generatedId = true)
    private Long id;
    @DatabaseField(foreign = true, foreignAutoRefresh = true)
    private Person person;
    @DatabaseField(dataType = DataType.ENUM_STRING)
    private Repeating repeating;

    public Period() {
    }

    /**
    * Constructor for period.
    * @param duration
    * @param begin
    * @param end
    * @param repeating
    */
    public Period(Integer duration, Date begin, Date end, Repeating repeating) {
        this.duration = duration;
        this.begin = begin;
        this.end = end;
        this.repeating = repeating;
    }

    public Date getBegin() {
        return begin;
    }

    public Integer getDuration() {
        return duration;
    }

    public Date getEnd() {
        return end;
    }

    public Long getId() {
        return this.id;
    }

    public Repeating getRepeating() {
        return repeating;
    }

    public void setBuilding(Building b) {
        this.building = b;
    }

    public void setEvent(Event e) {
        this.event = e;
    }

    public void setPerson(Person p) {
        this.person = p;
    }
}
