package de.uni_heidelberg.ise.studentnavigator.model;

import java.util.Collection;
import java.util.Locale;

import com.j256.ormlite.dao.ForeignCollection;
import com.j256.ormlite.field.DataType;
import com.j256.ormlite.field.DatabaseField;
import com.j256.ormlite.field.ForeignCollectionField;
import com.j256.ormlite.table.DatabaseTable;

import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.api.ISearchable;

@DatabaseTable(tableName = "persons")
@Feature("Routenfindung")
public class Person extends Organizer implements ISearchable {
    public enum Occupation {
        CARETAKER("HausmeisterIn"), DOCENT("DozentIn"), RESEARCH_ASSISTANT("wiss. MitarbeiterIn"), SECRETARY("SekretärIn"), TECHNICIAN(
                        "TechnikerIn"), TUTOR("TutorIn"), UNI_EXTERNAL("extern"), UNKNOWN("");

        private String name;

        private Occupation(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }
    }

    @DatabaseField
    private String email;

    @DatabaseField
    private String fax;
    @DatabaseField
    private String forename;
    @DatabaseField(dataType = DataType.ENUM_STRING)
    private Occupation occupation;
    @DatabaseField(foreign = true, foreignAutoRefresh = true)
    private Office office;
    @ForeignCollectionField(eager = true)
    private ForeignCollection<Period> officeHours;

    @DatabaseField(foreign = true, foreignAutoRefresh = true)
    private Organization organization;

    @DatabaseField
    private String phone;

    @DatabaseField
    private String surname;
    @DatabaseField
    private String title;
    @DatabaseField
    private String twitterUserName;

    @DatabaseField
    private String website;

    public Person() {
    }

    public Person(String forename, String surname, String title, String phone, String fax, String email, String website,
            Occupation occupation, Organization organization, Office office, String twitterUserName) {
        this.forename = forename;
        this.surname = surname;
        this.title = title;
        this.phone = phone;
        this.fax = fax;
        this.email = email;
        this.website = website;
        this.occupation = occupation;
        this.organization = organization;
        this.office = office;
        this.twitterUserName = twitterUserName;
    }

    @Override
    public boolean contains(String search) {
        return (getSearchableTitle() + " " + getSearchableDescription()).toLowerCase(Locale.ENGLISH)
                .contains(search.toLowerCase(Locale.ENGLISH));
    }

    public String getEmail() {
        return email;
    }

    public String getFax() {
        return fax;
    }

    public String getForename() {
        return forename;
    }

    @Override
    public MapObject getMapObject() {
        return getRoom();
    }

    public Occupation getOccupation() {
        return occupation;
    }

    public Collection<Period> getOfficeHours() {
        return officeHours;
    }

    public Organization getOrganization() {
        return organization;
    }

    @Override
    public String getOrganizerName() {
        return this.title + " " + this.forename + " " + this.surname;
    }

    public String getPhone() {
        return phone;
    }

    public Office getRoom() {
        return office;
    }

    @Override
    public String getSearchableDescription() {
        return getOccupation().getName() + (getOrganization() == null ? "" : ", " + getOrganization().getName())
                + (getRoom() == null || getRoom().getBuilding() == null ? ""
                        : ", " + getRoom().getBuilding().getName() + getRoom().getName());
    }

    @Override
    public String getSearchableTitle() {
        return getForename() + " " + getSurname();
    }

    public String getSurname() {
        return surname;
    }

    public String getTitle() {
        return title;
    }

    public String getTwitterUserName() {
        return twitterUserName;
    }

    public String getWebsite() {
        return website;
    }
}
