package de.uni_heidelberg.ise.studentnavigator.model.api;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.RuntimeExceptionDao;
import com.j256.ormlite.support.ConnectionSource;

import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Building;
import de.uni_heidelberg.ise.studentnavigator.model.Event;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.EventOrganization;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Organization;
import de.uni_heidelberg.ise.studentnavigator.model.Organizer;
import de.uni_heidelberg.ise.studentnavigator.model.Period;
import de.uni_heidelberg.ise.studentnavigator.model.Period.Repeating;
import de.uni_heidelberg.ise.studentnavigator.model.Person;
import de.uni_heidelberg.ise.studentnavigator.model.Person.Occupation;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet.Type;

public class DataManager implements IDataManager {
    private final ConnectionSource connectionSource;

    public DataManager(ConnectionSource connectionSource) {
        this.connectionSource = connectionSource;
    }

    @Override
    public Building addBuilding(String name, String address, String featureId) {
        try {
            final Building building = new Building(name, address, featureId);
            getDao(Building.class).createOrUpdate(building);
            return building;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Event addEvent(EventLocation eventLocation, String description, List<Organizer> organizer) {
        try {
            if (eventLocation.getId() == null) {
                eventLocation = addEventLocation(eventLocation.getName(), eventLocation.getBuilding(),
                        eventLocation.getOrganization(), eventLocation.getFeatureId());
            }

            final Event event = new Event(eventLocation, description);
            getDao(Event.class).createOrUpdate(event);

            if (organizer != null && organizer.size() > 0) {
                for (final Organizer org : organizer) {
                    getDao(EventOrganization.class).createOrUpdate(new EventOrganization(event, org));
                }
                event.setOrganizer(organizer);
            }

            return event;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public EventLocation addEventLocation(String name, Building building, Organization organization, String featureId) {
        try {
            if (building.getId() == null) {
                building = addBuilding(building.getName(), building.getAddress(), building.getFeatureId());
            }
            if (organization.getId() == null) {
                organization = addOrganization(organization.getName());
            }
            final EventLocation eventLocation = new EventLocation(name, building, organization, featureId);
            getDao(EventLocation.class).createOrUpdate(eventLocation);
            return eventLocation;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Office addOffice(String name, Building building, Organization organization, String featureId) {
        try {
            if (building.getId() == null) {
                building = addBuilding(building.getName(), building.getAddress(), building.getFeatureId());
            }
            if (organization.getId() == null) {
                organization = addOrganization(organization.getName());
            }
            final Office office = new Office(name, building, organization, featureId);
            getDao(Office.class).createOrUpdate(office);
            return office;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Organization addOrganization(String name) {
        try {
            final Organization organization = new Organization(name);
            getDao(Organization.class).createOrUpdate(organization);
            return organization;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Period addPeriod(Building building, Integer duration, Date begin, Date end, Repeating repeating) {
        try {
            if (building.getId() == null) {
                building = addBuilding(building.getName(), building.getAddress(), building.getFeatureId());
            }
            final Period period = new Period(duration, begin, end, repeating);
            period.setBuilding(building);
            getDao(Period.class).createOrUpdate(period);
            return period;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Period addPeriod(Event event, Integer duration, Date begin, Date end, Repeating repeating) {
        try {
            if (event.getId() == null) {
                event = addEvent(event.getEventLocation(), event.getDescription(), event.getOrganizer());
            }
            final Period period = new Period(duration, begin, end, repeating);
            period.setEvent(event);
            getDao(Period.class).createOrUpdate(period);
            return period;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Period addPeriod(Person person, Integer duration, Date begin, Date end, Repeating repeating) {
        try {
            if (person.getId() == null) {
                addPerson(person.getForename(), person.getSurname(), person.getTitle(), person.getPhone(),
                        person.getFax(), person.getEmail(), person.getWebsite(), person.getOccupation(),
                        person.getOrganization(), person.getRoom(), person.getTwitterUserName());
            }
            final Period period = new Period(duration, begin, end, repeating);
            period.setPerson(person);
            getDao(Period.class).createOrUpdate(period);
            return period;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * private List<Searchable> getSearchableRooms() { List<? extends Searchable> result = getBi; return
     * (List<Searchable>) result; }
     * 
     * private List<Searchable> getSearchableBuildings() { List<? extends Searchable> result = getBuildings(); return
     * (List<Searchable>) result; }
     */

    @Override
    public Person addPerson(String forename, String surname, String title, String phone, String fax, String email,
            String website, Occupation occupation, Organization organization, Office office, String twitterUserName) {
        try {
            final Dao<Person, Long> dao = getDao(Person.class);
            if (organization != null && organization.getId() == null) {
                organization = addOrganization(organization.getName());
            }
            if (office != null && office.getId() == null) {
                office = addOffice(office.getName(), office.getBuilding(), office.getOrganization(),
                        office.getFeatureId());
            }
            final Person person = new Person(forename, surname, title, phone, fax, email, website, occupation, organization,
                    office, twitterUserName);
            dao.createOrUpdate(person);
            return person;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Station addStation(String opnvId, String featureId, String longName, String shortName) {
        try {
            final Station station = new Station(opnvId, featureId, longName, shortName);
            getDao(Station.class).createOrUpdate(station);
            return station;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public Toilet addToilet(Type type, String name, Building building, Organization organization, String featureId) {
        try {
            if (building.getId() == null) {
                building = addBuilding(building.getName(), building.getAddress(), building.getFeatureId());
            }
            if (organization.getId() == null) {
                organization = addOrganization(organization.getName());
            }
            final Toilet toilet = new Toilet(type, name, building, organization, featureId);
            getDao(Toilet.class).createOrUpdate(toilet);
            return toilet;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public List<Building> getBuildings() {
        return queryForAll(Building.class);
    }

    private <T> Dao<T, Long> getDao(Class<T> clazz) throws SQLException {
        return RuntimeExceptionDao.createDao(connectionSource, clazz);
    }

    @Override
    public List<EventLocation> getEventLocations() {
        return queryForAll(EventLocation.class);
    }

    @Override
    public List<Event> getEvents() {
        try {
            final List<Event> tmpEvents = getDao(Event.class).queryForAll();
            if (tmpEvents != null && tmpEvents.size() > 0) {
                final ArrayList<Event> events = new ArrayList<Event>();
                for (final Event tmpEvent : tmpEvents) {
                    // get organizer of event
                    final List<EventOrganization> eventOrganizations = getDao(EventOrganization.class).queryForEq("event_id",
                            tmpEvent.getId());

                    final List<Organizer> tmpOrganizer = new ArrayList<Organizer>();

                    // fetch organization or person and add to organizer list
                    for (final EventOrganization eO : eventOrganizations) {
                        if (eO.getOrganization() != null) {
                            tmpOrganizer.add(getDao(Organization.class).queryForId(eO.getOrganization().getId()));
                        } else if (eO.getPerson() != null) {
                            tmpOrganizer.add(getDao(Person.class).queryForId(eO.getPerson().getId()));
                        }
                    }

                    // set prepared organizer list for event
                    tmpEvent.setOrganizer(tmpOrganizer);

                    // add event into result list
                    events.add(tmpEvent);
                }
                return events;
            }
            return tmpEvents;
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return new ArrayList<Event>();
    }

    @Override
    public List<Office> getOffices() {
        return queryForAll(Office.class);
    }

    @Override
    public List<Organization> getOrganizations() {
        return queryForAll(Organization.class);
    }

    @Override
    public List<Person> getPersons() {
        return queryForAll(Person.class);
    }

    @SuppressWarnings("unchecked")
    private List<ISearchable> getSearchableBuildings() {
        final List<? extends ISearchable> result = getBuildings();
        return (List<ISearchable>) result;
    }

    private List<ISearchable> getSearchablePersons() {
        final List<? extends ISearchable> resultPersons = getPersons();
        final List<ISearchable> result = new ArrayList<ISearchable>();
        result.addAll(resultPersons);
        return result;
    }

    private List<ISearchable> getSearchableRooms() {
        final List<? extends ISearchable> resultOffices = getOffices();
        final List<? extends ISearchable> resultToilets = getToilets();
        final List<? extends ISearchable> resultEventLocations = getEventLocations();

        final List<ISearchable> result = new ArrayList<ISearchable>();
        result.addAll(resultOffices);
        result.addAll(resultToilets);
        result.addAll(resultEventLocations);
        return result;
    }

    @Feature("opnv")
    @Override
    public List<Station> getStations() {
        return queryForAll(Station.class);
    }

    @Override
    public List<Toilet> getToilets() {
        return queryForAll(Toilet.class);
    }

    private <T> List<T> queryForAll(Class<T> t) {
        try {
            final Dao<T, Long> dao = getDao(t);
            return dao.queryForAll();
        } catch (final SQLException e) {
            e.printStackTrace();
        }
        return new ArrayList<T>();
    }

    @SuppressWarnings("unchecked")
	private List<ISearchable> getSearchableStations() {
    	final List<? extends ISearchable> result = getStations();
        return (List<ISearchable>) result;
    }
    
    @Override
    public List<ISearchable> search(String searchFor, String filter) {
        final List<ISearchable> result = new ArrayList<ISearchable>();
        final List<ISearchable> persons = getSearchablePersons();
        final List<ISearchable> rooms = getSearchableRooms();
        final List<ISearchable> buildings = getSearchableBuildings();
        final List<ISearchable> stations = getSearchableStations();

        final List<ISearchable> searchList = new ArrayList<ISearchable>();
        if (filter == "person") {
            searchList.addAll(persons);
        } else if (filter == "building") {
            searchList.addAll(buildings);
        } else if (filter == "room") {
            searchList.addAll(rooms);
        } else if( filter == "station") {
        	searchList.addAll(stations);
        }else {
            searchList.addAll(persons);
            searchList.addAll(buildings);
            searchList.addAll(rooms);
            searchList.addAll(stations);
        }

        for (final ISearchable searchable : searchList) {
            if (searchable.contains(searchFor)) {
                result.add(searchable);
            }
        }
        return result;
    }
}
