package de.uni_heidelberg.ise.studentnavigator.model.api;

import java.util.Date;
import java.util.List;

import de.uhd.ifi.se.feature.annotations.java.Feature;
import de.uni_heidelberg.ise.studentnavigator.model.Building;
import de.uni_heidelberg.ise.studentnavigator.model.Event;
import de.uni_heidelberg.ise.studentnavigator.model.EventLocation;
import de.uni_heidelberg.ise.studentnavigator.model.Office;
import de.uni_heidelberg.ise.studentnavigator.model.Organization;
import de.uni_heidelberg.ise.studentnavigator.model.Organizer;
import de.uni_heidelberg.ise.studentnavigator.model.Period;
import de.uni_heidelberg.ise.studentnavigator.model.Period.Repeating;
import de.uni_heidelberg.ise.studentnavigator.model.Person;
import de.uni_heidelberg.ise.studentnavigator.model.Person.Occupation;
import de.uni_heidelberg.ise.studentnavigator.model.Station;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet;
import de.uni_heidelberg.ise.studentnavigator.model.Toilet.Type;

public interface IDataManager {
    Building addBuilding(String name, String address, String featureId);

    Event addEvent(EventLocation l, String description, List<Organizer> os);

    EventLocation addEventLocation(String name, Building b, Organization o, String featureId);

    Office addOffice(String name, Building b, Organization o, String featureId);

    Organization addOrganization(String name);

    Period addPeriod(Building b, Integer duration, Date begin, Date end, Repeating r);

    Period addPeriod(Event e, Integer duration, Date begin, Date end, Repeating r);

    Period addPeriod(Person p, Integer duration, Date begin, Date end, Repeating r);

    Person addPerson(String forename, String surname, String title, String phone, String fax, String email,
            String website, Occupation o, Organization organization, Office office, String twitterUserName);

    Station addStation(String opnvId, String featureId, String longName, String shortName);

    Toilet addToilet(Type t, String name, Building b, Organization o, String featureId);

    List<Building> getBuildings();

    List<EventLocation> getEventLocations();

    List<Event> getEvents();

    List<Office> getOffices();

    List<Organization> getOrganizations();

    List<Person> getPersons();

    @Feature("opnv")
    List<Station> getStations();

    List<Toilet> getToilets();

    List<ISearchable> search(String searchFor, String filter);
}
