import xml.etree.ElementTree
import os
import sys
from xml.dom import minidom
from xml.etree.ElementTree import ElementTree, Element, SubElement, tostring
from networkx import *
from os import getcwd, listdir
from fnmatch import fnmatch
"""
:::description::: functions for source code structure based trace link improvements
"""


"""
:::description::: This function decicdes which cluster algorithm needs to be used. It is called by ui, therefore gets all req. data
:::param::: clustering paramters
:::return::: cluster list for ui to represent clusters
"""
def buildGraphFromXML(tr_set, git_path, scs_xml_path):
    #if (self.method == "function_call_xml"):
    print("Building Graph from XML file...")
    # create Graph
    G = nx.DiGraph()
    # parse xml file
    #xmldoc = minidom.parse(getcwd() + "\\xml_files\\source_code_graph.xml")
    xmldoc = minidom.parse(scs_xml_path)
    xml_branches = xmldoc.getElementsByTagName("javaElement")
    pattern = "*.java"
    for path, subdirs, files in os.walk(git_path):
        for name in files:
            if fnmatch(name, pattern):
                G.add_node(name)
                # get xml object subset of specific file name and append edges
                current_xml_branch = []
                for branch in xml_branches:
                    if str(branch.attributes['name'].value) == name:
                        current_xml_branch = branch
                try:
                    item_list = current_xml_branch.getElementsByTagName("dependency")
                    for item in item_list:
                        current_target = item.attributes['name'].value
                        current_target = current_target.rsplit('.', 1)[1] + ".java"
                        print(name + "---->" + current_target)
                        G.add_edge(name, current_target, weight=1.00)
                except:
                    print("no branch found, no edges created")
        # add edges from trace links
        for tl in tr_set.tracelinks:
            G.add_node(str(tl.bugid))
            print(str(tl.bugid) + "---->" + tl.target_path.rsplit('\\', 1)[1])
            G.add_edge(str(tl.bugid), tl.target_path.rsplit('\\', 1)[1], weight=round(tl.similarity, 3))
        print("Graph was built sucessfully")
        return G


"""
:::description::: This function returns the reconstructed graph graph used in the graph enhancement.
:::param:::
:::return::: directed graph
"""


def buildGraph(tr_set, git_path):
    print("Reconstructing Dependency Graph")
    #if (self.method == "function_call"):
    G = nx.DiGraph()
    # fill graph with nodes of code items
    pattern = "*.java"
    for path, subdirs, files in os.walk(git_path):
        for name in files:
            if fnmatch(name, pattern):
                # p = os.path.join(path, name)
                # n= GraphNode(name, p)
                G.add_node(name)
                content = []
                ##fill graph with edges from function calls
                with open(os.path.join(path, name)) as f:
                    content = f.readlines()
                    content = [x.strip() for x in content]
                for line in content:
                    if str(line).startswith('import org'):
                        f_name = line.replace(';', '').rsplit('.', 1)[1] + ".java"
                        print(name + "---->" + f_name)
                        G.add_edge(name, f_name, weight=1.00)

        # add edges from trace links
        for tl in tr_set.tracelinks:
            G.add_node(str(tl.bugid))
            print(str(tl.bugid) + "---->" + tl.target_path.rsplit('\\', 1)[1])
            G.add_edge(str(tl.bugid), tl.target_path.rsplit('\\', 1)[1], weight=round(tl.similarity, 3))
        print("Graph was built sucessfully")
        return G

def convert_path_2_class (path, prefix = 'StudentNavigator/src/'):
    # convert the workspace relative path of the source code file in 'source_file' to the fully
    # qualified class name of the respective java class
    path = path[len(prefix):].replace('/','.')
    return path[:path.find('.java')]

def get_path4_class_name(class_name, src_files):

    for src_file in src_files:
        if class_name.replace('.', '/') in src_file:
            return src_file



def add_code_dep_links(tr_set, src_files, code_graph_xml, search_level_deep):
    # extend the interaction links with dependencies to classes found by the static source code structure
    dg = xml.etree.ElementTree.parse(code_graph_xml)
    scs_links = []
    print("%d links before code dep links added" % len(tr_set))
    for tr in tr_set :
        # can only process java
        if tr[1].endswith('.java'):
            # find the dependencies for a java class name
            for dep in find_dependencies(dg, convert_path_2_class(tr[1]), search_level_deep):
                # go through all source code files
                for src_file in src_files:
                    # check if there is a source code file for the found dependency
                    if dep.replace('.', '/') in src_file:
                        # avoid links pointing to the src file itself
                        if src_file != tr[1]:
                            # create the source code structure link and add it to the list of source code
                            # structure links
                            scs_link = [tr[0], src_file]
                            #print("scs link: %s --> %s" % (tr[1], src_file, dep))
                            if scs_link not in scs_links:
                                scs_links.append(scs_link)

    for scs_link in scs_links:
        if scs_link not in tr_set:
            #print("NEW: %s --> %s" % (scs_link[0], scs_link[1]))
            tr_set.append(scs_link)
        #else:
        #    print("existing: %s --> %s" % (scs_link[0], scs_link[1]))
    print("%d links after code dep links added" % len(tr_set))



def find_dependencies(in_tree, fqc_name, search_level_deep, r_level=0):
    """
    :param in_tree: source code graph / tree as parsed from generated jdt xml
    :param fqc_name: fully qualified class name for which to search the dependencies
    :param r_level: actual number of recursion level with which the method is called, gets incremented by the
    method itself until self.search_level_deep is reached
    :return: a list of fully qualified class names of the dependencies
    """
    deps = []
    for project in in_tree.findall('project'):
        # print(project.items())
        for package in project.findall('package'):
            # print("  %s" % package.get('name'))
            # if fqc_name in package.findall('javaElement'):
            for je in package.findall('javaElement'):
                # fully qualified java element name consisting of :
                # package name + class name  without '.java'
                # e.g. 'org.eclipse.mylyn.web.core.WebCorePlugin'
                f_je = package.get('name') + '.' + je.get('name')[:je.get('name').find('.java')]
                # print("%s==%s" % (f_je,fqc_name))
                if f_je == fqc_name:
                    for dep in je.findall('dependency'):
                        # only add found dependencies if the recursion call level is below the nesting deep aka
                        # serch level
                        if r_level < search_level_deep:
                            dep_name = dep.get('name')
                            # FIXME: just removing the $ part of enum names is too simple one solution might be replacing $ with
                            # . and also create a node element of each type with a $ in its name
                            if '$' in dep_name:
                                dep_name = dep_name[:dep_name.find('$')]
                            deps.append(dep_name)
                            # if r_level <= self.search_level_deep:
                            # if r_level < self.search_level_deep:
                            r_level += 1
                            deps.extend(find_dependencies(in_tree, dep_name, search_level_deep, r_level))
                        # else:
                        #    break
                    # break
    return deps